@@ -1,4 +1,3 @@
-/* $XFree86: xc/lib/GL/mesa/src/drv/r128/r128_state.c,v 1.11 2002/10/30 12:51:39 alanh Exp $ */
 /**************************************************************************
 
 Copyright 1999, 2000 ATI Technologies Inc. and Precision Insight, Inc.,
@@ -40,16 +39,15 @@ USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "r128_tris.h"
 #include "r128_tex.h"
 
-#include "context.h"
-#include "enums.h"
-#include "colormac.h"
+#include "main/context.h"
+#include "main/enums.h"
+#include "main/colormac.h"
+#include "main/macros.h"
 #include "swrast/swrast.h"
 #include "vbo/vbo.h"
 #include "tnl/tnl.h"
 #include "swrast_setup/swrast_setup.h"
 
-#include "tnl/t_pipeline.h"
-
 #include "drirenderbuffer.h"
 
 
@@ -127,7 +125,7 @@ static int blend_factor( r128ContextPtr rmesa, GLenum factor, GLboolean is_src )
 }
 
 
-static void r128UpdateAlphaMode( GLcontext *ctx )
+static void r128UpdateAlphaMode( struct gl_context *ctx )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
    GLuint a = rmesa->setup.misc_3d_state_cntl_reg;
@@ -211,7 +209,7 @@ static void r128UpdateAlphaMode( GLcontext *ctx )
    }
 }
 
-static void r128DDAlphaFunc( GLcontext *ctx, GLenum func, GLfloat ref )
+static void r128DDAlphaFunc( struct gl_context *ctx, GLenum func, GLfloat ref )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
 
@@ -219,7 +217,7 @@ static void r128DDAlphaFunc( GLcontext *ctx, GLenum func, GLfloat ref )
    rmesa->new_state |= R128_NEW_ALPHA;
 }
 
-static void r128DDBlendEquationSeparate( GLcontext *ctx, 
+static void r128DDBlendEquationSeparate( struct gl_context *ctx, 
 					 GLenum modeRGB, GLenum modeA )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
@@ -241,7 +239,7 @@ static void r128DDBlendEquationSeparate( GLcontext *ctx,
    rmesa->new_state |= R128_NEW_ALPHA;
 }
 
-static void r128DDBlendFuncSeparate( GLcontext *ctx,
+static void r128DDBlendFuncSeparate( struct gl_context *ctx,
 				     GLenum sfactorRGB, GLenum dfactorRGB,
 				     GLenum sfactorA, GLenum dfactorA )
 {
@@ -256,7 +254,7 @@ static void r128DDBlendFuncSeparate( GLcontext *ctx,
  */
 
 static void
-r128DDStencilFuncSeparate( GLcontext *ctx, GLenum face, GLenum func,
+r128DDStencilFuncSeparate( struct gl_context *ctx, GLenum face, GLenum func,
                            GLint ref, GLuint mask )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
@@ -304,7 +302,7 @@ r128DDStencilFuncSeparate( GLcontext *ctx, GLenum face, GLenum func,
 }
 
 static void
-r128DDStencilMaskSeparate( GLcontext *ctx, GLenum face, GLuint mask )
+r128DDStencilMaskSeparate( struct gl_context *ctx, GLenum face, GLuint mask )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
    GLuint refmask = (((ctx->Stencil.Ref[0] & 0xff) << 0) |
@@ -317,7 +315,7 @@ r128DDStencilMaskSeparate( GLcontext *ctx, GLenum face, GLuint mask )
    }
 }
 
-static void r128DDStencilOpSeparate( GLcontext *ctx, GLenum face, GLenum fail,
+static void r128DDStencilOpSeparate( struct gl_context *ctx, GLenum face, GLenum fail,
                                      GLenum zfail, GLenum zpass )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
@@ -416,7 +414,7 @@ static void r128DDStencilOpSeparate( GLcontext *ctx, GLenum face, GLenum fail,
    }
 }
 
-static void r128DDClearStencil( GLcontext *ctx, GLint s )
+static void r128DDClearStencil( struct gl_context *ctx, GLint s )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
 
@@ -430,7 +428,7 @@ static void r128DDClearStencil( GLcontext *ctx, GLint s )
  * Depth testing
  */
 
-static void r128UpdateZMode( GLcontext *ctx )
+static void r128UpdateZMode( struct gl_context *ctx )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
    GLuint z = rmesa->setup.z_sten_cntl_c;
@@ -487,7 +485,7 @@ static void r128UpdateZMode( GLcontext *ctx )
    }
 }
 
-static void r128DDDepthFunc( GLcontext *ctx, GLenum func )
+static void r128DDDepthFunc( struct gl_context *ctx, GLenum func )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
 
@@ -495,7 +493,7 @@ static void r128DDDepthFunc( GLcontext *ctx, GLenum func )
    rmesa->new_state |= R128_NEW_DEPTH;
 }
 
-static void r128DDDepthMask( GLcontext *ctx, GLboolean flag )
+static void r128DDDepthMask( struct gl_context *ctx, GLboolean flag )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
 
@@ -503,7 +501,7 @@ static void r128DDDepthMask( GLcontext *ctx, GLboolean flag )
    rmesa->new_state |= R128_NEW_DEPTH;
 }
 
-static void r128DDClearDepth( GLcontext *ctx, GLclampd d )
+static void r128DDClearDepth( struct gl_context *ctx, GLclampd d )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
 
@@ -526,7 +524,7 @@ static void r128DDClearDepth( GLcontext *ctx, GLclampd d )
  * Fog
  */
 
-static void r128UpdateFogAttrib( GLcontext *ctx )
+static void r128UpdateFogAttrib( struct gl_context *ctx )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
    GLuint t = rmesa->setup.tex_cntl_c;
@@ -555,7 +553,7 @@ static void r128UpdateFogAttrib( GLcontext *ctx )
    }
 }
 
-static void r128DDFogfv( GLcontext *ctx, GLenum pname, const GLfloat *param )
+static void r128DDFogfv( struct gl_context *ctx, GLenum pname, const GLfloat *param )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
 
@@ -568,12 +566,12 @@ static void r128DDFogfv( GLcontext *ctx, GLenum pname, const GLfloat *param )
  * Clipping
  */
 
-static void r128UpdateClipping( GLcontext *ctx )
+static void r128UpdateClipping( struct gl_context *ctx )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
 
    if ( rmesa->driDrawable ) {
-      __DRIdrawablePrivate *drawable = rmesa->driDrawable;
+      __DRIdrawable *drawable = rmesa->driDrawable;
       int x1 = 0;
       int y1 = 0;
       int x2 = drawable->w - 1;
@@ -624,7 +622,7 @@ static void r128UpdateClipping( GLcontext *ctx )
    }
 }
 
-static void r128DDScissor( GLcontext *ctx,
+static void r128DDScissor( struct gl_context *ctx,
 			   GLint x, GLint y, GLsizei w, GLsizei h )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
@@ -638,7 +636,7 @@ static void r128DDScissor( GLcontext *ctx,
  * Culling
  */
 
-static void r128UpdateCull( GLcontext *ctx )
+static void r128UpdateCull( struct gl_context *ctx )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
    GLuint f = rmesa->setup.pm4_vc_fpu_setup;
@@ -677,7 +675,7 @@ static void r128UpdateCull( GLcontext *ctx )
    }
 }
 
-static void r128DDCullFace( GLcontext *ctx, GLenum mode )
+static void r128DDCullFace( struct gl_context *ctx, GLenum mode )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
 
@@ -685,7 +683,7 @@ static void r128DDCullFace( GLcontext *ctx, GLenum mode )
    rmesa->new_state |= R128_NEW_CULL;
 }
 
-static void r128DDFrontFace( GLcontext *ctx, GLenum mode )
+static void r128DDFrontFace( struct gl_context *ctx, GLenum mode )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
 
@@ -698,15 +696,15 @@ static void r128DDFrontFace( GLcontext *ctx, GLenum mode )
  * Masks
  */
 
-static void r128UpdateMasks( GLcontext *ctx )
+static void r128UpdateMasks( struct gl_context *ctx )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
 
    GLuint mask = r128PackColor( rmesa->r128Screen->cpp,
-				ctx->Color.ColorMask[RCOMP],
-				ctx->Color.ColorMask[GCOMP],
-				ctx->Color.ColorMask[BCOMP],
-				ctx->Color.ColorMask[ACOMP] );
+				ctx->Color.ColorMask[0][RCOMP],
+				ctx->Color.ColorMask[0][GCOMP],
+				ctx->Color.ColorMask[0][BCOMP],
+				ctx->Color.ColorMask[0][ACOMP] );
 
    if ( rmesa->setup.plane_3d_mask_c != mask ) {
       rmesa->setup.plane_3d_mask_c = mask;
@@ -714,7 +712,7 @@ static void r128UpdateMasks( GLcontext *ctx )
    }
 }
 
-static void r128DDColorMask( GLcontext *ctx,
+static void r128DDColorMask( struct gl_context *ctx,
 			     GLboolean r, GLboolean g,
 			     GLboolean b, GLboolean a )
 {
@@ -733,7 +731,7 @@ static void r128DDColorMask( GLcontext *ctx,
  * sense to break them out of the core texture state update routines.
  */
 
-static void updateSpecularLighting( GLcontext *ctx )
+static void updateSpecularLighting( struct gl_context *ctx )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
    GLuint t = rmesa->setup.tex_cntl_c;
@@ -763,7 +761,7 @@ static void updateSpecularLighting( GLcontext *ctx )
 }
 
 
-static void r128DDLightModelfv( GLcontext *ctx, GLenum pname,
+static void r128DDLightModelfv( struct gl_context *ctx, GLenum pname,
 				const GLfloat *param )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
@@ -772,9 +770,14 @@ static void r128DDLightModelfv( GLcontext *ctx, GLenum pname,
       FLUSH_BATCH( rmesa );
       updateSpecularLighting(ctx);
    }
+
+   if ( pname == GL_LIGHT_MODEL_TWO_SIDE ) {
+      FLUSH_BATCH( rmesa );
+      r128ChooseRenderState( ctx );
+   }
 }
 
-static void r128DDShadeModel( GLcontext *ctx, GLenum mode )
+static void r128DDShadeModel( struct gl_context *ctx, GLenum mode )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
    GLuint s = rmesa->setup.pm4_vc_fpu_setup;
@@ -808,7 +811,7 @@ static void r128DDShadeModel( GLcontext *ctx, GLenum mode )
  * Window position
  */
 
-static void r128UpdateWindow( GLcontext *ctx )
+static void r128UpdateWindow( struct gl_context *ctx )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
    int x = rmesa->driDrawable->x;
@@ -831,7 +834,7 @@ static void r128UpdateWindow( GLcontext *ctx )
  * Viewport
  */
 
-static void r128CalcViewport( GLcontext *ctx )
+static void r128CalcViewport( struct gl_context *ctx )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
    const GLfloat *v = ctx->Viewport._WindowMap.m;
@@ -847,14 +850,14 @@ static void r128CalcViewport( GLcontext *ctx )
    m[MAT_TZ] =   v[MAT_TZ] * rmesa->depth_scale;
 }
 
-static void r128Viewport( GLcontext *ctx,
+static void r128Viewport( struct gl_context *ctx,
 			  GLint x, GLint y,
 			  GLsizei width, GLsizei height )
 {
    r128CalcViewport( ctx );
 }
 
-static void r128DepthRange( GLcontext *ctx,
+static void r128DepthRange( struct gl_context *ctx,
 			    GLclampd nearval, GLclampd farval )
 {
    r128CalcViewport( ctx );
@@ -865,7 +868,7 @@ static void r128DepthRange( GLcontext *ctx,
  * Miscellaneous
  */
 
-static void r128DDClearColor( GLcontext *ctx,
+static void r128DDClearColor( struct gl_context *ctx,
 			      const GLfloat color[4] )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
@@ -880,7 +883,7 @@ static void r128DDClearColor( GLcontext *ctx,
 				      c[0], c[1], c[2], c[3] );
 }
 
-static void r128DDLogicOpCode( GLcontext *ctx, GLenum opcode )
+static void r128DDLogicOpCode( struct gl_context *ctx, GLenum opcode )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
 
@@ -891,7 +894,7 @@ static void r128DDLogicOpCode( GLcontext *ctx, GLenum opcode )
    }
 }
 
-static void r128DDDrawBuffer( GLcontext *ctx, GLenum mode )
+static void r128DDDrawBuffer( struct gl_context *ctx, GLenum mode )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
 
@@ -918,7 +921,7 @@ static void r128DDDrawBuffer( GLcontext *ctx, GLenum mode )
    rmesa->new_state |= R128_NEW_WINDOW;
 }
 
-static void r128DDReadBuffer( GLcontext *ctx, GLenum mode )
+static void r128DDReadBuffer( struct gl_context *ctx, GLenum mode )
 {
    /* nothing, until we implement h/w glRead/CopyPixels or CopyTexImage */
 }
@@ -928,7 +931,7 @@ static void r128DDReadBuffer( GLcontext *ctx, GLenum mode )
  * Polygon stipple
  */
 
-static void r128DDPolygonStipple( GLcontext *ctx, const GLubyte *mask )
+static void r128DDPolygonStipple( struct gl_context *ctx, const GLubyte *mask )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
    GLuint stipple[32], i;
@@ -959,7 +962,7 @@ static void r128DDPolygonStipple( GLcontext *ctx, const GLubyte *mask )
  * Render mode
  */
 
-static void r128DDRenderMode( GLcontext *ctx, GLenum mode )
+static void r128DDRenderMode( struct gl_context *ctx, GLenum mode )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
    FALLBACK( rmesa, R128_FALLBACK_RENDER_MODE, (mode != GL_RENDER) );
@@ -971,7 +974,7 @@ static void r128DDRenderMode( GLcontext *ctx, GLenum mode )
  * State enable/disable
  */
 
-static void r128DDEnable( GLcontext *ctx, GLenum cap, GLboolean state )
+static void r128DDEnable( struct gl_context *ctx, GLenum cap, GLboolean state )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
 
@@ -1203,7 +1206,7 @@ static void r128DDPrintState( const char *msg, GLuint flags )
 	    (flags & R128_NEW_WINDOW)	? "window, " : "" );
 }
 
-void r128DDUpdateHWState( GLcontext *ctx )
+void r128DDUpdateHWState( struct gl_context *ctx )
 {
    r128ContextPtr rmesa = R128_CONTEXT(ctx);
    int new_state = rmesa->new_state;
@@ -1250,7 +1253,7 @@ void r128DDUpdateHWState( GLcontext *ctx )
 }
 
 
-static void r128DDInvalidateState( GLcontext *ctx, GLuint new_state )
+static void r128DDInvalidateState( struct gl_context *ctx, GLuint new_state )
 {
    _swrast_InvalidateState( ctx, new_state );
    _swsetup_InvalidateState( ctx, new_state );
@@ -1401,17 +1404,15 @@ void r128DDInitState( r128ContextPtr rmesa )
 
 /* Initialize the driver's state functions.
  */
-void r128DDInitStateFuncs( GLcontext *ctx )
+void r128DDInitStateFuncs( struct gl_context *ctx )
 {
    ctx->Driver.UpdateState		= r128DDInvalidateState;
 
-   ctx->Driver.ClearIndex		= NULL;
    ctx->Driver.ClearColor		= r128DDClearColor;
    ctx->Driver.ClearStencil		= r128DDClearStencil;
    ctx->Driver.DrawBuffer		= r128DDDrawBuffer;
    ctx->Driver.ReadBuffer		= r128DDReadBuffer;
 
-   ctx->Driver.IndexMask		= NULL;
    ctx->Driver.ColorMask		= r128DDColorMask;
    ctx->Driver.AlphaFunc		= r128DDAlphaFunc;
    ctx->Driver.BlendEquationSeparate	= r128DDBlendEquationSeparate;
